#!/bin/sh
   
#   Copyright (C) 2003 CSIRO Australia
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#   
#   - Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#   
#   - Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#   
#   - Neither the name of the CSIRO nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#   
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
#   PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE ORGANISATION OR
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
#   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
#  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# Testing script for the mod_annodex.php cgi

anx_mime="application/x-annodex"
cmml_mime="text/x-cmml"

# Success and failure functions
failures=0
successes=0
total=0
fail() {
	failures=`expr $failures + 1`
	total=`expr $total + 1`
	echo "$total:	FAILED -> $1"
	return
}
succeed() {
	successes=`expr $failures + 1`
	total=`expr $total + 1`
	echo "$total:	Succeeded -> $1"
	return
}


# CHECK USAGE 

usage="Usage: $0 <Annodex URL>"
if [ $# -ne 1 ]; then
	echo 1>&2 $usage
	exit 127
fi
url=$1

# check URL works
head=`curl $url --head --silent`
if [ $? -ne 0 ]; then
	echo "URL must specify a valid Annodex."
	echo "Terminating script."
	exit 1
fi


# HEADER TESTS

echo "***Header Tests***"
anx_head_regexp="HTTP/1.1 200 OK
Date: .*
Server: .*
X-Content-Bitrate-Average: ?-[0-9]*?\.[0-9]*
X-Content-Duration: [0-9]*?\.[0-9]*
Content-Type: $anx_mime" 

cmml_head_regexp="HTTP/1.1 200 OK
Date: .*
Server: .*
Content-Length: .*
Content-Type: $cmml_mime" 


# anx header well formed?
pos=`expr match "$head" "$anx_head_regexp"`
if [ ! $pos ]; then
	fail "Annodex header badly formed: '$head'" 
else
	succeed "Annodex header fine" 
fi

# cmml header well formed?
head=`curl $url --head --silent --header \"Accept: text/x-cmml\"` 2> /dev/null
pos=`expr match "$head" "$cmml_head_regexp"`
if [ ! $pos ]; then
	fail "CMML header badly formed" 
else
	succeed "CMML header fine" 
fi

echo ""

# OGG VALIDATION TESTS
echo "***Ogg Validation Tests***"
ogg="ogg.temp"
curl --silent $url > $ogg

# ogg valid?
oggz-validate $ogg
if [ $? -ne 0 ]; then
	fail "Invalid Ogg"
else
	succeed "Valid Ogg"
fi

echo ""

# CMML TESTS
echo "***CMML Tests***"
cmml="cmml.temp"
curl $url --silent --header "Accept: text/x-cmml" > $cmml 

# cmml valid?
error=`cmml-validate $cmml 2>&1 | cat`
if [ "$error" ]; then
	fail "Invalid CMML"
else
	succeed "Valid CMML"
fi

# cmml same as anxripped?
ripped_cmml="ripped_cmml.temp"
#anxrip $ogg > $ripped_cmml 2> /dev/null
#diffs=`diff -w $cmml $ripped_cmml`
#if [ "$diffs" ]; then
#	fail "Ripped CMML different from served CMML"
#else
#	succeed "Ripped CMML matches served CMML"
#fi

echo ""

# OFFSET TESTS


# WRAP UP

# clean
rm -f $ripped_cmml
rm -f $ogg
rm -f $cmml

# print results
echo ""
if [ $failures -ne 1 ]; then
	plural="s"
fi
echo "$failures failure$plural from $total tests."

# Only total success deserves a success exit value 
if [ $failures -ne 0 ]; then
	exit 1
else
	echo "All test cases passed!"
	exit 0
fi



