#!/usr/bin/env php 
<?php

/*
 *  Copyright (C) 2006 CSIRO Australia
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  - Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *  
 *  - Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  
 *  - Neither the name of the CSIRO nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 *  PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE ORGANISATION OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
  require_once("phpannodex/AnnodexWriter.php");
  require_once("phpannodex/AnxInterval/AnxInterval.php");

  $filename_root = get_filename_root();

  /* If application/x-cmml is a higher priority than 
   * application/x-annodex then we want to output cmml. Otherwise we want
   * to output annodex.
   */
   if( cmml_prioritised( $_SERVER['HTTP_ACCEPT'] ) ) {
    output_cmml( $filename_root );
  } else {
    output_annodex( $filename_root );
  }
  exit;

function get_filename_root() {

  /* FIXME There must be a better way of getting the actual path of the
   * file than this. Also, I'm not sure that this will port at all.
   */
  $filename = $_ENV['DOCUMENT_ROOT'] . substr($_SERVER['PATH_INFO'],1);

  /* Remove ".anx" */
  $filename_root = substr( $filename, 0, strlen($filename) - 
              strlen(".anx" ) );
  return $filename_root;
}
  
/* Check the accept field of the request. If cmml appears before annodex return
 * true, otherwise return false.
 */
function cmml_prioritised( $http_accept ) {

  /* Break up the accept field of the request so we can parse it. */
  $accept_array = explode(",",$http_accept);

  foreach( $accept_array as $type ) {
    $type_tuple = explode(";",$type);
    if( $type_tuple[0]=="text/x-cmml" ) {
      /* Found cmml before annodex. */
      if( count($type_tuple) == 2 ) {
        $cmml_priority = $type_tuple[1];
      } else {
        $cmml_priority = 1;
      }
    } else if ( $accept_array[$key][1]=="application/x-annodex" ) {
      /* Found annodex before cmml. */
      if( count($type_tuple) == 2 ) {
        $anx_priority = $type_tuple[1];
      } else {
        $anx_priority = 1;
      }
    }
  }

  if( $cmml_priority > $anx_priority ) {
    return true;
  } else {
    return false;
  }
}

/* Output the boring header stuff. Does not specify Content-Type, as 
 * output_cmml() and o )utput_annodex() do that.
 */
function output_headers( $anx=false ) {

  if( !$anx ) {
    echo("Content-Type: text/x-cmml\r\n");
  } else {
    echo "Content-Type: ", AnnodexWriter::MIME_TYPE, "\n";
    /* Output bitrate and duration information to the headers. */
    echo "X-Content-Bitrate-Average: ", $anx->get_bitrate(), "\r\n";
    echo "X-Content-Duration: ", $anx->get_duration(), "\r\n";
  }
  echo "\r\n";
  return;
}
  
/* Output the requested .anx file as cmml. */
function output_cmml( $filename_root ) {

  output_headers(); 

  /* Strip the "anx" off the filename and replace it with "cmml".
         * This should be done in a more elegant way
   */
  $cmml_filename = $filename_root . ".cmml";
  
  /* Spit it straight out, removing the src attribute from imports. */
  $xml = simplexml_load_file( $cmml_filename );
  $dom = dom_import_simplexml( $xml );
  $dom->removeAttribute("stream");
  $stream = $dom->getElementsByTagName("stream");
  $dom->removeChild( $stream->item( 0 ) );
  //foreach( $xml->clip as $clip ) {
  //  $clip['track']="default";
  //}
  $xml = simplexml_import_dom( $dom );
  echo $xml->asXML();

  return;
}

/* Output the requested .anx file as proper annodex. */
function output_annodex( $filename_root ) {

  $filename = $filename_root . ".ogg";
  $cmml_filename = $filename_root . ".cmml";
  $anx_filename = $filename_root . ".anx";

  try {
    $anx = new AnnodexWriter();
  } catch( Exception $e ) {
    print $e->getTraceAsString(); 
  }


  parse_str( $_SERVER['QUERY_STRING'], $query_array );

  /* Parse the temporal bit of the URI */
  $interval = AnxInterval::factory($query_array['t']);

  /* Grab clip and id parameters if they are supplied. */
  $id = $query_array['id'];
  $clip = $query_array['clip'];
  #echo "id is $id";
  try {
    $anx->import( $cmml_filename, $id, $content_type, 
      $interval->get_start(), $interval->get_end());
  } catch ( AnxException $e ) {
    echo( $e->getMessage()."\n" );
    exit;
  }

  output_headers( $anx );

  /* Output the file as a series of binary strings, the size of which
         * is determined by BUFFER_SIZE.  
   */
  while( $buffer = $anx->write_output() ) {
    echo $buffer;
  }
  //var_dump( $anx->get_anx_handle() );

  $anx->close();

  return;
}

?>
